package net.lapisblue.server.images;

import java.awt.*;
import java.awt.image.*;
import java.text.*;
import java.io.*;
import java.util.*;
import javax.imageio.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class ArrowServlet extends ImageServlet {
  class ArrowGenerator extends ImageGenerator {
    final int size;
    final int stroke;
    final Color color;
    final Color fill;
    final String direction;
    final String mode;
    final String linestyle;
    
    ArrowGenerator (HttpServletRequest request) {
      super(request);

      this.stroke = getInt("s");
      this.size = getInt("z");
      this.color = getColor("c");
      this.fill = getColor("f");
      this.direction = getString("d");
      this.mode = getString("m");
      this.linestyle = getString("l");
    }

    protected int getImageWidth () { return size; }
    protected int getImageHeight () { return size; }
    
    protected void draw (Graphics2D g2d) {
      g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                           RenderingHints.VALUE_ANTIALIAS_ON);

      g2d.setColor(color);
      
      if ("dash".equals(linestyle)) {
        g2d.setStroke(new BasicStroke(stroke, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER,
                                      2f * stroke, new float[]{3f*stroke, 3f*stroke}, 1.5f*stroke));
      } else if ("dot".equals(linestyle)) {
        g2d.setStroke(new BasicStroke(stroke, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER,
                                      2f * stroke, new float[]{stroke, stroke}, 0.5f * stroke));
      } else {
        g2d.setStroke(new BasicStroke(stroke));
      }
      
      if ("ns".equals(direction)) {
        g2d.drawLine(size/2, size, size/2, 0);
      } else if ("ew".equals(direction)) {
        g2d.drawLine(0, size/2, size, size/2);
      } else if ("ne".equals(direction)) {
        g2d.drawOval(size/2, -size/2, size, size);
      } else if ("se".equals(direction)) {
        g2d.drawOval(size/2, size/2, size, size);
      } else if ("nw".equals(direction)) {
        g2d.drawOval(-size/2, -size/2, size, size);
      } else if ("sw".equals(direction)) {
        g2d.drawOval(-size/2, size/2, size, size);
      } else if ("new".equals(direction)) {
        g2d.drawLine(0, size/2, size, size/2);
        g2d.drawLine(size/2, 0, size/2, size/2);
      } else if ("sew".equals(direction)) {
        g2d.drawLine(0, size/2, size, size/2);
        g2d.drawLine(size/2, size/2, size/2, size);
      } else if ("nse".equals(direction)) {
        g2d.drawLine(size/2, size/2, size, size/2);
        g2d.drawLine(size/2, size, size/2, 0);
      } else if ("nsw".equals(direction)) {
        g2d.drawLine(0, size/2, size/2, size/2);
        g2d.drawLine(size/2, size, size/2, 0);
      } else if ("nsew".equals(direction)) {
        g2d.drawLine(0, size/2, size, size/2);
        g2d.drawLine(size/2, size, size/2, 0);
      } else {
        if ("w".equals(direction)) {
          g2d.rotate(Math.PI, size/2, size/2);
        } else if ("n".equals(direction)) { 
          g2d.rotate(-Math.PI/2, size/2, size/2);
        } else if ("s".equals(direction)) { 
          g2d.rotate(Math.PI/2, size/2, size/2);
        }
        
        final String mode = this.mode == null ? "closed" : this.mode;
        final int mid = size/2;
        
        if (mode.startsWith("circle")) {
          g2d.setColor(fill);
          g2d.fillOval(stroke, stroke, size - 2*stroke - 1, size - 2*stroke - 1);
          g2d.setColor(color);
          g2d.drawLine(0, mid, stroke - 1, mid);
          g2d.setStroke(new BasicStroke(stroke));
          g2d.drawOval(stroke, stroke, size - 2*stroke - 1, size - 2*stroke - 1);
          if ("circle-bar".equals(mode)) {
            g2d.drawLine(mid, 4*stroke, mid, size - 4*stroke);
          } else if ("circle-plus".equals(mode)) {
            g2d.drawLine(mid, 4*stroke, mid, size - 4*stroke);
            g2d.drawLine(4*stroke, mid, size - 4*stroke, mid);
          } else if ("circle-minus".equals(mode)) {
            g2d.drawLine(4*stroke, mid, size - 4*stroke, mid);
          } else if ("circle-star".equals(mode)) {
            g2d.drawLine(mid, 4*stroke, mid, size - 4*stroke);
            g2d.rotate(Math.PI/3, mid, mid);
            g2d.drawLine(mid, 4*stroke, mid, size - 4*stroke);
            g2d.rotate(Math.PI/3, mid, mid);
            g2d.drawLine(mid, 4*stroke, mid, size - 4*stroke);
          } else if ("circle-dot".equals(mode)) {
            final int r = size/6;
            g2d.fillOval(mid - r, mid - r, 2*r - 1, 2*r - 1);
          }
        } else if ("cup".equals(mode)) {
          g2d.drawLine(0, mid, stroke - 1, mid);
          g2d.setStroke(new BasicStroke(stroke));
          g2d.drawArc(stroke, stroke, size - 2*stroke - 1, size - 2*stroke - 1, 70, 220);
        } else if (mode.startsWith("cup-circle")) {
          final int r = size/2 - 3*stroke;
          g2d.setColor(fill);
          g2d.fillOval(mid - r, mid - r, 2*r - 1, 2*r - 1);
          g2d.setColor(color);
          g2d.drawLine(0, mid, stroke - 1, mid);
          g2d.drawLine(size-3*stroke + 1, mid, size, mid);
          g2d.setStroke(new BasicStroke(stroke));
          g2d.drawArc(stroke, stroke, size - 2*stroke - 1, size - 2*stroke - 1, 70, 220);
          g2d.drawOval(mid - r, mid - r, 2*r - 1, 2*r - 1);
        } else if (mode.startsWith("lollipop")) {
          final int r = size/2 - 3*stroke;
          g2d.setColor(fill);
          g2d.fillOval(mid - r, mid - r, 2*r - 1, 2*r - 1);
          g2d.setColor(color);
          g2d.drawLine(size-3*stroke + 1, mid, size, mid);
          g2d.setStroke(new BasicStroke(stroke));
          g2d.drawOval(mid - r, mid - r, 2*r - 1, 2*r - 1);
        } else if ("bar".equals(mode)) {
          g2d.drawLine(0, mid, size, mid);
          g2d.setStroke(new BasicStroke(stroke));
          g2d.drawLine(mid, 4*stroke, mid, size - 4*stroke);
        } else if ("grave".equals(mode)) {
          g2d.drawLine(0, mid, size, mid);
          g2d.setStroke(new BasicStroke(stroke));
          g2d.rotate(Math.PI/6, mid, mid);
          g2d.drawLine(mid, 4*stroke, mid, size - 4*stroke);
        } else if ("stroke".equals(mode)) {
          g2d.drawLine(0, mid, size, mid);
          g2d.setStroke(new BasicStroke(stroke));
          g2d.rotate(-Math.PI/6, mid, mid);
          g2d.drawLine(mid, 4*stroke, mid, size - 4*stroke);
        } else if ("x".equals(mode)) {
          g2d.drawLine(0, mid, mid, mid);
          g2d.setStroke(new BasicStroke(stroke));
          g2d.drawLine(4*stroke, 4*stroke, size - 4*stroke, size - 4*stroke);
          g2d.drawLine(4*stroke, size - 4*stroke, size - 4*stroke, 4*stroke);
        } else if ("star".equals(mode)) {
          g2d.drawLine(0, mid, size, mid);
          g2d.setStroke(new BasicStroke(stroke));
          g2d.drawLine(mid, 4*stroke, mid, size - 4*stroke);
          g2d.rotate(Math.PI/3, mid, mid);
          g2d.drawLine(mid, 4*stroke, mid, size - 4*stroke);
          g2d.rotate(Math.PI/3, mid, mid);
          g2d.drawLine(mid, 4*stroke, mid, size - 4*stroke);
        } else if ("open".equals(mode)) {
          g2d.drawLine(0, mid, size - 2*stroke, mid);
          g2d.setStroke(new BasicStroke(stroke));
          g2d.drawPolyline(
            new int[]{size/6, size - stroke, size/6},
            new int[]{mid - size/4, mid, mid + size/4}, 3);
        } else if ("closed".equals(mode)) {
          g2d.setColor(fill);
          g2d.fillPolygon(
            new int[]{size/6, size - stroke, size/6},
            new int[]{mid - size/4, mid, mid + size/4}, 3);
          g2d.setColor(color);
          g2d.drawLine(0, mid, size/6, mid);
          g2d.setStroke(new BasicStroke(stroke));
          g2d.drawPolygon(
            new int[]{size/6, size - stroke, size/6},
            new int[]{mid - size/4, mid, mid + size/4}, 3);
        } else if ("wide-open".equals(mode)) {
          g2d.drawLine(0, mid, size - 2*stroke, mid);
          g2d.setStroke(new BasicStroke(stroke));
          g2d.drawPolyline(
            new int[]{size/6, size - stroke, size/6},
            new int[]{stroke, mid, size -stroke}, 3);
        } else if ("wide".equals(mode)) {
          g2d.setColor(fill);
          g2d.fillPolygon(
            new int[]{size/6, size - stroke, size/6},
            new int[]{stroke, mid, size -stroke}, 3);
          g2d.setColor(color);
          g2d.drawLine(0, mid, size/6, mid);
          g2d.setStroke(new BasicStroke(stroke));
          g2d.drawPolygon(
            new int[]{size/6, size - stroke, size/6},
            new int[]{stroke, mid, size -stroke}, 3);
        } else if (mode.startsWith("rhombus")) {
          g2d.setColor(fill);
          g2d.fillPolygon(
            new int[]{stroke, mid, size - stroke, mid},
            new int[]{mid, size-stroke, mid, stroke}, 4);
          g2d.setColor(color);
          g2d.drawLine(0, mid, stroke, mid);
          g2d.setStroke(new BasicStroke(stroke));
          g2d.drawPolygon(
            new int[]{stroke, mid, size - stroke, mid},
            new int[]{mid, size-stroke, mid, stroke}, 4);
        } else if (mode.startsWith("diamond")) {
          g2d.setColor(fill);
          g2d.fillPolygon(
            new int[]{stroke, mid, size - stroke, mid},
            new int[]{mid, size/4, mid, mid + size/4}, 4);
          g2d.setColor(color);
          g2d.drawLine(0, mid, stroke, mid);
          g2d.setStroke(new BasicStroke(stroke));
          g2d.drawPolygon(
            new int[]{stroke, mid, size - stroke, mid},
            new int[]{mid, size/4, mid, mid + size/4}, 4);
        }
      }
    }
  }
    
  protected ImageGenerator createImageGenerator (HttpServletRequest request) {
    return new ArrowGenerator(request);
  }
}